/*
 *  TOPPERS ATK2
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *      Automotive Kernel Version 2
 *
 *  Copyright (C) 2008-2017 by Center for Embedded Computing Systems
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2011-2017 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2011-2013 by Spansion LLC, USA
 *  Copyright (C) 2011-2017 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2011-2016 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2011-2014 by Renesas Electronics Corporation, JAPAN
 *  Copyright (C) 2011-2016 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2011-2017 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2011-2017 by Witz Corporation
 *  Copyright (C) 2014-2016 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2014-2016 by eSOL Co.,Ltd., JAPAN
 *  Copyright (C) 2014-2017 by SCSK Corporation, JAPAN
 *  Copyright (C) 2015-2017 by SUZUKI MOTOR CORPORATION
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: target_test.h 2401 2017-03-14 09:09:24Z witz-itoyo $
 */

/*
 *		テストプログラムのターゲット依存定義（NIOS2_DEV用）
 */

#ifndef TOPPERS_TARGET_TEST_H
#define TOPPERS_TARGET_TEST_H

/*
 *  sample1で使用する例外
 */
#define RAISE_CPU_EXCEPTION	Asm("div zero, zero, zero")

/*
 *  sample1で使用するアラームの周期（約1秒）
 */
#define COUNTER_MIN_CYCLE	((uint32) 60000000)

#define SYSCYC_TIME			((uint32) 0xe4e1c00)    /* システム周期 */
#define TWD2_ALARM_TIME		((uint32) 0x66ff300)    /* TWD2のオフセット */
#define TWDD_ALARM_TIME		((uint32) 0xcdfe600)    /* アイドルWDのオフセット */

/*
 *  sample1で使用する割込み要求
 */
LOCAL_INLINE void
Sample1RaiseInterrupt(void)
{
	/* タイマーストップ */
	sil_wrw_iop((void *) TIMER_CONTROL(TIMER_5_IRQ), TIMER_CONTROL_STOP);
	/* タイムアウトステータスクリア */
	sil_wrw_iop((void *) TIMER_STATUS(TIMER_5_IRQ), 0x00U);
	/* タイマカウンターセット 上位16bit */
	sil_wrw_iop((void *) TIMER_PERIODH(TIMER_5_IRQ), 0x00);
	/* タイマカウンターセット 下位16bit */
	sil_wrw_iop((void *) TIMER_PERIODL(TIMER_5_IRQ), 0x01);
	/* タイマースタート，割込み許可 */
	sil_wrw_iop((void *) TIMER_CONTROL(TIMER_5_IRQ), TIMER_CONTROL_START | TIMER_CONTROL_ITO);
}

/*
 *  sample1で使用する割込み要求のクリア
 */
LOCAL_INLINE void
Sample1ClearIntRequest(void)
{
	/* TOビットのクリア */
	sil_wrw_iop((void *) TIMER_STATUS(TIMER_5_IRQ), 0x00U);
}

/*
 * sample1.yamlで使用する割込み番号
 */
#define TPTEST_INT_NUM	(21)

#ifndef TOPPERS_MACRO_ONLY

extern void trigger_hwcounter_SmpC1ISR(void);
extern void c1isr_syslog(void);

#endif /* TOPPERS_MACRO_ONLY */

#endif /* TOPPERS_TARGET_TEST_H */
